@@ -43,7 +43,7 @@ from log import *
 # How slack expresses mentioning users
 _MENTIONS_REGEXP = re.compile(r'<@([0-9A-Za-z]+)>')
 _CHANNEL_MENTIONS_REGEXP = re.compile(r'<#[A-Z0-9]+\|([A-Z0-9\-a-z]+)>')
-_URL_REGEXP=re.compile(r'<([a-z0-9\-\.]+)://([^\s\|]+)[\|]{0,1}([^<>]*)>')
+_URL_REGEXP = re.compile(r'<([a-z0-9\-\.]+)://([^\s\|]+)[\|]{0,1}([^<>]*)>')
 
 
 _SLACK_SUBSTITUTIONS = [
@@ -58,7 +58,13 @@ class Replies(Enum):
     RPL_USERHOST = 302
     RPL_UNAWAY = 305
     RPL_NOWAWAY = 306
+    RPL_WHOISUSER = 311
+    RPL_WHOISSERVER = 312
+    RPL_WHOISOPERATOR = 313
     RPL_ENDOFWHO = 315
+    RPL_WHOISIDLE = 317
+    RPL_ENDOFWHOIS = 318
+    RPL_WHOISCHANNELS = 319
     RPL_LIST = 322
     RPL_LISTEND = 323
     RPL_CHANNELMODEIS = 324
@@ -66,6 +72,7 @@ class Replies(Enum):
     RPL_WHOREPLY = 352
     RPL_NAMREPLY = 353
     RPL_ENDOFNAMES = 366
+    ERR_NOSUCHNICK = 401
     ERR_NOSUCHCHANNEL = 403
     ERR_UNKNOWNCOMMAND = 421
     ERR_FILEERROR = 424
@@ -96,6 +103,8 @@ class Client:
         self.autojoin = autojoin
         self._usersent = False # Used to hold all events until the IRC client sends the initial USER message
         self._held_events: List[slack.SlackEvent] = []
+        self._magic_users_id = 0
+        self._magic_regex: Optional[re.Pattern] = None
 
         if self.provider == Provider.SLACK:
             self.substitutions = _SLACK_SUBSTITUTIONS
@@ -298,6 +307,31 @@ class Client:
         except Exception:
             self._sendreply(Replies.ERR_UNKNOWNCOMMAND, f'Unable to set topic to {topic}')
 
+    def _whoishandler(self, cmd: bytes) -> None:
+        users = cmd.split(b' ')
+        del users[0]
+
+        if len(users) > 1:
+            self._sendreply(Replies.ERR_UNKNOWNCOMMAND, 'Server parameter is not supported')
+
+        # Seems that oftc only responds to the last one
+        username = users.pop()
+
+        if b'*' in username:
+            self._sendreply(Replies.ERR_UNKNOWNCOMMAND, 'Wildcards are not supported')
+        uusername = username.decode()
+        try:
+            user = self.sl_client.get_user_by_name(uusername)
+        except KeyError:
+            self._sendreply(Replies.ERR_NOSUCHNICK, f'Unknown user {uusername}')
+
+        self._sendreply(Replies.RPL_WHOISUSER, user.real_name, [username, '', 'localhost'])
+        if user.profile.email:
+            self._sendreply(Replies.RPL_WHOISUSER, f'email: {user.profile.email}', [username, '', 'localhost'])
+        if user.is_admin:
+            self._sendreply(Replies.RPL_WHOISOPERATOR, f'{uusername} is an IRC operator', [username])
+        self._sendreply(Replies.RPL_ENDOFWHOIS, '', extratokens=[username])
+
     def _kickhandler(self, cmd: bytes) -> None:
         _, channel_b, username, message = cmd.split(b' ', 3)
         channel = self.sl_client.get_channel_by_name(channel_b.decode()[1:])
@@ -373,14 +407,29 @@ class Client:
 
         # Extremely inefficient code to generate mentions
         # Just doing them client-side on the receiving end is too mainstream
-        for username in self.sl_client.get_usernames():
-            if len(username) < 3: continue
-            m = re.search(r'\b%s\b' % username, msg)
-            if m:
-                if self.provider == Provider.SLACK:
-                    msg = msg[0:m.start()] + '<@%s>' % self.sl_client.get_user_by_name(username).id + msg[m.end():]
-                elif self.provider == Provider.ROCKETCHAT:
-                    msg = msg[0:m.start()] + f'@{username}' + msg[m.end():]
+
+
+        if self._magic_users_id == id(self.sl_client.get_usernames()):
+            regex = self._magic_regex
+            assert regex
+        else:
+            usernames = self.sl_client.get_usernames()
+            assert usernames
+            self._magic_users_id = id(usernames)
+            regexs = (r'((://\S*){0,1}\b%s\b)' % username for username in usernames)
+            regex = re.compile('|'.join(regexs))
+            self._magic_regex = regex
+
+        matches = list(re.finditer(regex, msg))
+        matches.reverse() # I want to replace from end to start or the positions get broken
+        for m in matches:
+            username = m.string[m.start():m.end()]
+            if username.startswith('://'):
+                continue # Match inside a url
+            elif self.provider == Provider.SLACK:
+                msg = msg[0:m.start()] + '<@%s>' % self.sl_client.get_user_by_name(username).id + msg[m.end():]
+            elif self.provider == Provider.ROCKETCHAT:
+                msg = msg[0:m.start()] + f'@{username}' + msg[m.end():]
         return msg
 
     def parse_message(self, msg: str) -> Iterator[bytes]:
@@ -538,7 +587,7 @@ class Client:
             #QUIT
             #CAP LS
             b'USERHOST': self._userhosthandler,
-            #Unknown command:  b'whois TAMARRO'
+            b'whois': self._whoishandler,
         }
 
         if cmdid in handlers:
