@@ -29,8 +29,9 @@ from websocket import create_connection, WebSocket
 from websocket._exceptions import WebSocketConnectionClosedException
 import typedload.dataloader
 
-from slack import Channel, File, FileShared, IM, Message, MessageEdit, Profile, SlackEvent, Topic, User
+from slack import Channel, File, FileShared, IM, Message, MessageEdit, Profile, SlackEvent, Topic, User, NoChanMessage
 from slackclient.client import Team, Self, LoginInfo
+from log import *
 
 CALL_TIMEOUT = 10
 
@@ -58,7 +59,7 @@ class DirectChannel(NamedTuple):
 
 
 class Rocket:
-    def __init__(self, url: str, token: str) -> None:
+    def __init__(self, url: str, token: str, previous_status: Optional[bytes]) -> None:
         self.url = url
         self.token  = token
         self._call_id = 100
@@ -80,8 +81,11 @@ class Rocket:
         self.loader.handlers[index] = (self.loader.handlers[index][0], _namedtuplehandler)
         self._connect()
 
+    def get_status(self) -> bytes:
+        return b''
+
     @property
-    def login_info(self):
+    def login_info(self) -> LoginInfo:
         #TODO
         return LoginInfo(
             team=Team(
@@ -131,7 +135,7 @@ class Rocket:
             elif channel_type == ChannelType.QUERY:
                 pass
             else:
-                print('Unknown data %s' % repr(i))
+                log('Unknown data %s' % repr(i))
 
     def _send_json(self, data: Dict[str, Any]) -> None:
         """
@@ -214,6 +218,9 @@ class Rocket:
     def away(self, is_away: bool) -> None:
         raise NotImplemented()
 
+    def join(self, channel: Channel) -> None:
+        raise NotImplemented()
+
     def invite(self, channel: Channel, user: Union[User, List[User]]) -> None:
         raise NotImplemented()
 
@@ -229,7 +236,9 @@ class Rocket:
                 )
         return {i.id_ for i in data.records}
 
-    def channels(self) -> List[Channel]:
+    def channels(self, refresh: bool=False) -> List[Channel]:
+        if refresh:
+            self._update_channels()
         return self._channels
 
     def get_channel(self, id_: str) -> Channel:
@@ -295,18 +304,18 @@ class Rocket:
         try:
             _, raw_data = self._websocket.recv_data()
             if raw_data == b'\x03\xe8Normal closure':
-                print('Server triggered a disconnect. Reaconnecting')
+                log('Server triggered a disconnect. Reaconnecting')
                 raise Exception('Trigger reconnect')
         except SSLWantReadError:
             return None
-        except:
+        except Exception:
             self._connect()
             return None
 
         try:
             data = json.loads(raw_data)
-        except:
-            print(f'Failed to decode json: {repr(raw_data)}')
+        except Exception:
+            log(f'Failed to decode json: {repr(raw_data)}')
             raise
 
 
@@ -340,7 +349,7 @@ class Rocket:
                 continue
 
             r: Optional[SlackEvent] = None
-            print('Scanning ', data)
+            log('Scanning ', data)
             if not isinstance(data, dict):
                 continue
 
@@ -348,7 +357,7 @@ class Rocket:
             try:
                 if data['fields']['args'][0]['_id'].startswith(self._id_prefix):
                     continue
-            except:
+            except Exception:
                 pass
 
             if data.get('msg') == 'changed' and data.get('collection') == 'stream-room-messages': # New message
@@ -367,13 +376,16 @@ class Rocket:
                     )
                     if 'editedBy' in data['fields']['args'][0]:
                         r = MessageEdit(
-                            previous=Message(channel=r.channel, user=r.user, text=''),
-                            current=r
+                            type='message',
+                            subtype='message_changed',
+                            channel=r.channel,
+                            previous=NoChanMessage(user=r.user, text=''),
+                            current=NoChanMessage(user=r.user, text=r.text)
                         )
-                except:
+                except Exception:
                     pass
 
             if r is None:
-                print('Not handled: ', data)
+                log('Not handled: ', data)
             else:
                 yield r
