@@ -487,7 +487,7 @@ class Slack:
         if not response.ok:
             raise ResponseException(response)
 
-    def get_members(self, id_: str) -> Set[str]:
+    def get_members(self, channel: Union[str, Channel]) -> Set[str]:
         """
         Returns the list (as a set) of users in a channel.
 
@@ -497,6 +497,11 @@ class Slack:
 
         When events happen, the cache needs to be updated or cleared.
         """
+        if isinstance(channel, Channel):
+            id_ = channel.id
+        else:
+            id_ = channel
+
         cached = self._get_members_cache.get(id_, set())
         cursor = self._get_members_cache_cursor.get(id_)
         if cursor == '':
@@ -606,6 +611,7 @@ class Slack:
     def get_user_by_name(self, name: str) -> User:
         return self._usermapcache[name]
 
+    @lru_cache
     def get_usernames(self) -> List[str]:
         return list(self._usermapcache.keys())
 
@@ -619,6 +625,7 @@ class Slack:
             for user in load(r['members'], List[User]):
                 self._usercache[user.id] = user
                 self._usermapcache[user.name] = user
+            self.get_usernames.cache_clear()
 
     def get_user(self, id_: str) -> User:
         """
@@ -634,6 +641,8 @@ class Slack:
         if response.ok:
             u = load(r['user'], User)
             self._usercache[id_] = u
+            if u.name not in self._usermapcache:
+                self.get_usernames.cache_clear()
             self._usermapcache[u.name] = u
             return u
         else:
@@ -780,9 +789,9 @@ class Slack:
                         continue
 
                 try:
-                    ev = load(
+                    ev: Optional[Union[TopicChange, FileShared, MessageBot, MessageEdit, MessageDelete, GroupJoined, Join, Leave]] = load(
                         event,
-                        Union[TopicChange, FileShared, MessageBot, MessageEdit, MessageDelete, GroupJoined, Join, Leave]
+                        Union[TopicChange, FileShared, MessageBot, MessageEdit, MessageDelete, GroupJoined, Join, Leave]  # type: ignore
                     )
                 except Exception:
                     ev = None
